#include <iostream>
#include <string>
#include <algorithm>

class BST_KeyValue {
private:
    struct Node {
        std::string key;
        int value;
        Node* left;
        Node* right;
        Node(const std::string& k, int v) : key(k), value(v), left(nullptr), right(nullptr) {}
    };

    Node* root;

    Node* insert(Node* node, const std::string& key, int value) {
        if (node == nullptr) {
            return new Node(key, value);
        }

        if (key < node->key) {
            node->left = insert(node->left, key, value);
        } else if (key > node->key) {
            node->right = insert(node->right, key, value);
        } else {
            // Update value if key already exists
            node->value = value;
        }

        return node;
    }

    Node* search(Node* node, const std::string& key) const {
        if (node == nullptr || node->key == key) {
            return node;
        }

        if (key < node->key) {
            return search(node->left, key);
        }
        return search(node->right, key);
    }

    void inorderTraversal(Node* node) const {
        if (node) {
            inorderTraversal(node->left);
            std::cout << node->key << ": " << node->value << std::endl;
            inorderTraversal(node->right);
        }
    }

    void destroyTree(Node* node) {
        if (node) {
            destroyTree(node->left);
            destroyTree(node->right);
            delete node;
        }
    }

public:
    BST_KeyValue() : root(nullptr) {}
    ~BST_KeyValue() { destroyTree(root); }

    void insert(const std::string& key, int value) {
        root = insert(root, key, value);
    }

    bool search(const std::string& key, int& value) const {
        Node* result = search(root, key);
        if (result) {
            value = result->value;
            return true;
        }
        return false;
    }

    void printInOrder() const {
        inorderTraversal(root);
    }
};

int main() {
    BST_KeyValue bst;

    bst.insert("apple", 5);
    bst.insert("banana", 7);
    bst.insert("cherry", 3);
    bst.insert("date", 1);
    bst.insert("elderberry", 2);

    std::cout << "Key-Value pairs in order:" << std::endl;
    bst.printInOrder();

    std::string searchKey = "banana";
    int value;
    if (bst.search(searchKey, value)) {
        std::cout << "Value for key '" << searchKey << "': " << value << std::endl;
    } else {
        std::cout << "Key '" << searchKey << "' not found." << std::endl;
    }

    // Update a value
    bst.insert("banana", 10);

    std::cout << "\nAfter updating 'banana':" << std::endl;
    bst.printInOrder();

    return 0;
}
